// A little test program to make sure the BeShare 2.22
// "send this Message to the MUSCLE server" function works.
// This program will tell the BeShare user with the specified
// session ID to connect back to BeShare... if it works,
// you'll see a pink box show up in BeShare's transfer pane!
//
// Compile thus:  gcc testsendmsg.cpp
//
// Usage:  testsendmsg <userid> [port]
//

#include <stdio.h>
#include <app/Message.h>
#include <app/Messenger.h>

#include "ShareConstants.h"
#include "reflector/StorageReflectConstants.h"

int
main (int argc, char **argv)
{
	if (argc < 2)
	{
		printf ("Usage:  testsendmsg <userid> [port=7000]\n");
		exit (10);
	}

	const char *userID = argv[1];
	int port = (argc > 2) ? atoi (argv[2]) : 7000;

	BMessage dataMessage (0x03); // NET_CLIENT_CONNECT_BACK_REQUEST
	char buf[256];
	sprintf (buf, "/*/%s/beshare", userID);
	dataMessage.AddString (PR_NAME_KEYS, buf);
	dataMessage.AddString (PR_NAME_SESSION, "blah");	// muscle server will change this to the correct value
	dataMessage.AddInt32 ("port", port);

	BMessage msg ('smsg');
	msg.AddMessage ("message", &dataMessage);

	status_t error;
	BMessenger toBeShare (BESHARE_MIME_TYPE, -1, &error);
	if (error == B_NO_ERROR)
	{
		if (toBeShare.SendMessage (&msg) == B_NO_ERROR)
		{
			printf ("Send Message (%s:%i) to BeShare!\n", userID, port);
		}
		else
			printf ("SendMessage() failed!?\n");
	}
	else
		printf ("BeShare isn't running?\n");

	return 0;
}
